# Warning: this file is auto-generated. Do not edit.

VERSION_1_0 = 1
MIN_COMPOSITION_LAYERS_SUPPORTED = 16
NULL_HANDLE = 0
NULL_SYSTEM_ID = 0
NULL_PATH = 0
NO_DURATION = 0
INFINITE_DURATION = 0x7fffffffffffffff
MIN_HAPTIC_DURATION = -1
FREQUENCY_UNSPECIFIED = 0
TRUE = 1
FALSE = 0
MAX_EXTENSION_NAME_SIZE = 128
MAX_API_LAYER_NAME_SIZE = 256
MAX_API_LAYER_DESCRIPTION_SIZE = 256
MAX_SYSTEM_NAME_SIZE = 256
MAX_APPLICATION_NAME_SIZE = 128
MAX_ENGINE_NAME_SIZE = 128
MAX_RUNTIME_NAME_SIZE = 128
MAX_PATH_LENGTH = 256
MAX_STRUCTURE_NAME_SIZE = 64
MAX_RESULT_STRING_SIZE = 64
MAX_ACTION_SET_NAME_SIZE = 64
MAX_LOCALIZED_ACTION_SET_NAME_SIZE = 128
MAX_ACTION_NAME_SIZE = 64
MAX_LOCALIZED_ACTION_NAME_SIZE = 128
KHR_composition_layer_cube = 1
KHR_composition_layer_cube_SPEC_VERSION = 8
KHR_COMPOSITION_LAYER_CUBE_EXTENSION_NAME = "XR_KHR_composition_layer_cube"
KHR_composition_layer_depth = 1
KHR_composition_layer_depth_SPEC_VERSION = 6
KHR_COMPOSITION_LAYER_DEPTH_EXTENSION_NAME = "XR_KHR_composition_layer_depth"
KHR_composition_layer_cylinder = 1
KHR_composition_layer_cylinder_SPEC_VERSION = 4
KHR_COMPOSITION_LAYER_CYLINDER_EXTENSION_NAME = "XR_KHR_composition_layer_cylinder"
KHR_composition_layer_equirect = 1
KHR_composition_layer_equirect_SPEC_VERSION = 3
KHR_COMPOSITION_LAYER_EQUIRECT_EXTENSION_NAME = "XR_KHR_composition_layer_equirect"
KHR_visibility_mask = 1
KHR_visibility_mask_SPEC_VERSION = 2
KHR_VISIBILITY_MASK_EXTENSION_NAME = "XR_KHR_visibility_mask"
KHR_composition_layer_color_scale_bias = 1
KHR_composition_layer_color_scale_bias_SPEC_VERSION = 5
KHR_COMPOSITION_LAYER_COLOR_SCALE_BIAS_EXTENSION_NAME = "XR_KHR_composition_layer_color_scale_bias"
KHR_loader_init = 1
KHR_loader_init_SPEC_VERSION = 1
KHR_LOADER_INIT_EXTENSION_NAME = "XR_KHR_loader_init"
KHR_composition_layer_equirect2 = 1
KHR_composition_layer_equirect2_SPEC_VERSION = 1
KHR_COMPOSITION_LAYER_EQUIRECT2_EXTENSION_NAME = "XR_KHR_composition_layer_equirect2"
KHR_binding_modification = 1
KHR_binding_modification_SPEC_VERSION = 1
KHR_BINDING_MODIFICATION_EXTENSION_NAME = "XR_KHR_binding_modification"
KHR_swapchain_usage_input_attachment_bit = 1
KHR_swapchain_usage_input_attachment_bit_SPEC_VERSION = 3
KHR_SWAPCHAIN_USAGE_INPUT_ATTACHMENT_BIT_EXTENSION_NAME = "XR_KHR_swapchain_usage_input_attachment_bit"
EXT_performance_settings = 1
EXT_performance_settings_SPEC_VERSION = 3
EXT_PERFORMANCE_SETTINGS_EXTENSION_NAME = "XR_EXT_performance_settings"
EXT_thermal_query = 1
EXT_thermal_query_SPEC_VERSION = 2
EXT_THERMAL_QUERY_EXTENSION_NAME = "XR_EXT_thermal_query"
EXT_debug_utils = 1
EXT_debug_utils_SPEC_VERSION = 4
EXT_DEBUG_UTILS_EXTENSION_NAME = "XR_EXT_debug_utils"
EXT_eye_gaze_interaction = 1
EXT_eye_gaze_interaction_SPEC_VERSION = 2
EXT_EYE_GAZE_INTERACTION_EXTENSION_NAME = "XR_EXT_eye_gaze_interaction"
EXTX_overlay = 1
EXTX_overlay_SPEC_VERSION = 5
EXTX_OVERLAY_EXTENSION_NAME = "XR_EXTX_overlay"
VARJO_quad_views = 1
VARJO_quad_views_SPEC_VERSION = 1
VARJO_QUAD_VIEWS_EXTENSION_NAME = "XR_VARJO_quad_views"
MSFT_unbounded_reference_space = 1
MSFT_unbounded_reference_space_SPEC_VERSION = 1
MSFT_UNBOUNDED_REFERENCE_SPACE_EXTENSION_NAME = "XR_MSFT_unbounded_reference_space"
MSFT_spatial_anchor = 1
MSFT_spatial_anchor_SPEC_VERSION = 2
MSFT_SPATIAL_ANCHOR_EXTENSION_NAME = "XR_MSFT_spatial_anchor"
FB_composition_layer_image_layout = 1
FB_composition_layer_image_layout_SPEC_VERSION = 1
FB_COMPOSITION_LAYER_IMAGE_LAYOUT_EXTENSION_NAME = "XR_FB_composition_layer_image_layout"
FB_composition_layer_alpha_blend = 1
FB_composition_layer_alpha_blend_SPEC_VERSION = 2
FB_COMPOSITION_LAYER_ALPHA_BLEND_EXTENSION_NAME = "XR_FB_composition_layer_alpha_blend"
MND_headless = 1
MND_headless_SPEC_VERSION = 2
MND_HEADLESS_EXTENSION_NAME = "XR_MND_headless"
OCULUS_android_session_state_enable = 1
OCULUS_android_session_state_enable_SPEC_VERSION = 1
OCULUS_ANDROID_SESSION_STATE_ENABLE_EXTENSION_NAME = "XR_OCULUS_android_session_state_enable"
EXT_view_configuration_depth_range = 1
EXT_view_configuration_depth_range_SPEC_VERSION = 1
EXT_VIEW_CONFIGURATION_DEPTH_RANGE_EXTENSION_NAME = "XR_EXT_view_configuration_depth_range"
EXT_conformance_automation = 1
EXT_conformance_automation_SPEC_VERSION = 3
EXT_CONFORMANCE_AUTOMATION_EXTENSION_NAME = "XR_EXT_conformance_automation"
MSFT_spatial_graph_bridge = 1
MSFT_spatial_graph_bridge_SPEC_VERSION = 2
MSFT_SPATIAL_GRAPH_BRIDGE_EXTENSION_NAME = "XR_MSFT_spatial_graph_bridge"
GUID_SIZE_MSFT = 16
MSFT_hand_interaction = 1
MSFT_hand_interaction_SPEC_VERSION = 1
MSFT_HAND_INTERACTION_EXTENSION_NAME = "XR_MSFT_hand_interaction"
EXT_hand_tracking = 1
HAND_JOINT_COUNT_EXT = 26
EXT_hand_tracking_SPEC_VERSION = 4
EXT_HAND_TRACKING_EXTENSION_NAME = "XR_EXT_hand_tracking"
MSFT_hand_tracking_mesh = 1
MSFT_hand_tracking_mesh_SPEC_VERSION = 4
MSFT_HAND_TRACKING_MESH_EXTENSION_NAME = "XR_MSFT_hand_tracking_mesh"
MSFT_secondary_view_configuration = 1
MSFT_secondary_view_configuration_SPEC_VERSION = 1
MSFT_SECONDARY_VIEW_CONFIGURATION_EXTENSION_NAME = "XR_MSFT_secondary_view_configuration"
MSFT_first_person_observer = 1
MSFT_first_person_observer_SPEC_VERSION = 1
MSFT_FIRST_PERSON_OBSERVER_EXTENSION_NAME = "XR_MSFT_first_person_observer"
MSFT_controller_model = 1
NULL_CONTROLLER_MODEL_KEY_MSFT = 0
MSFT_controller_model_SPEC_VERSION = 2
MSFT_CONTROLLER_MODEL_EXTENSION_NAME = "XR_MSFT_controller_model"
MAX_CONTROLLER_MODEL_NODE_NAME_SIZE_MSFT = 64
EXT_win32_appcontainer_compatible = 1
EXT_win32_appcontainer_compatible_SPEC_VERSION = 1
EXT_WIN32_APPCONTAINER_COMPATIBLE_EXTENSION_NAME = "XR_EXT_win32_appcontainer_compatible"
EPIC_view_configuration_fov = 1
EPIC_view_configuration_fov_SPEC_VERSION = 2
EPIC_VIEW_CONFIGURATION_FOV_EXTENSION_NAME = "XR_EPIC_view_configuration_fov"
MSFT_composition_layer_reprojection = 1
MSFT_composition_layer_reprojection_SPEC_VERSION = 1
MSFT_COMPOSITION_LAYER_REPROJECTION_EXTENSION_NAME = "XR_MSFT_composition_layer_reprojection"
HUAWEI_controller_interaction = 1
HUAWEI_controller_interaction_SPEC_VERSION = 1
HUAWEI_CONTROLLER_INTERACTION_EXTENSION_NAME = "XR_HUAWEI_controller_interaction"
FB_swapchain_update_state = 1
FB_swapchain_update_state_SPEC_VERSION = 3
FB_SWAPCHAIN_UPDATE_STATE_EXTENSION_NAME = "XR_FB_swapchain_update_state"
FB_composition_layer_secure_content = 1
FB_composition_layer_secure_content_SPEC_VERSION = 1
FB_COMPOSITION_LAYER_SECURE_CONTENT_EXTENSION_NAME = "XR_FB_composition_layer_secure_content"
EXT_dpad_binding = 1
EXT_dpad_binding_SPEC_VERSION = 1
EXT_DPAD_BINDING_EXTENSION_NAME = "XR_EXT_dpad_binding"
VALVE_analog_threshold = 1
VALVE_analog_threshold_SPEC_VERSION = 2
VALVE_ANALOG_THRESHOLD_EXTENSION_NAME = "XR_VALVE_analog_threshold"
EXT_hand_joints_motion_range = 1
EXT_hand_joints_motion_range_SPEC_VERSION = 1
EXT_HAND_JOINTS_MOTION_RANGE_EXTENSION_NAME = "XR_EXT_hand_joints_motion_range"
EXT_samsung_odyssey_controller = 1
EXT_samsung_odyssey_controller_SPEC_VERSION = 1
EXT_SAMSUNG_ODYSSEY_CONTROLLER_EXTENSION_NAME = "XR_EXT_samsung_odyssey_controller"
EXT_hp_mixed_reality_controller = 1
EXT_hp_mixed_reality_controller_SPEC_VERSION = 1
EXT_HP_MIXED_REALITY_CONTROLLER_EXTENSION_NAME = "XR_EXT_hp_mixed_reality_controller"
MND_swapchain_usage_input_attachment_bit = 1
MND_swapchain_usage_input_attachment_bit_SPEC_VERSION = 2
MND_SWAPCHAIN_USAGE_INPUT_ATTACHMENT_BIT_EXTENSION_NAME = "XR_MND_swapchain_usage_input_attachment_bit"
MSFT_scene_understanding = 1
MSFT_scene_understanding_SPEC_VERSION = 2
MSFT_SCENE_UNDERSTANDING_EXTENSION_NAME = "XR_MSFT_scene_understanding"
MSFT_scene_understanding_serialization = 1
MSFT_scene_understanding_serialization_SPEC_VERSION = 2
MSFT_SCENE_UNDERSTANDING_SERIALIZATION_EXTENSION_NAME = "XR_MSFT_scene_understanding_serialization"
FB_display_refresh_rate = 1
FB_display_refresh_rate_SPEC_VERSION = 1
FB_DISPLAY_REFRESH_RATE_EXTENSION_NAME = "XR_FB_display_refresh_rate"
HTC_vive_cosmos_controller_interaction = 1
HTC_vive_cosmos_controller_interaction_SPEC_VERSION = 1
HTC_VIVE_COSMOS_CONTROLLER_INTERACTION_EXTENSION_NAME = "XR_HTC_vive_cosmos_controller_interaction"
HTCX_vive_tracker_interaction = 1
HTCX_vive_tracker_interaction_SPEC_VERSION = 1
HTCX_VIVE_TRACKER_INTERACTION_EXTENSION_NAME = "XR_HTCX_vive_tracker_interaction"
HTC_facial_tracking = 1
FACIAL_EXPRESSION_EYE_COUNT_HTC = 14
FACIAL_EXPRESSION_LIP_COUNT_HTC = 37
HTC_facial_tracking_SPEC_VERSION = 1
HTC_FACIAL_TRACKING_EXTENSION_NAME = "XR_HTC_facial_tracking"
HTC_vive_focus3_controller_interaction = 1
HTC_vive_focus3_controller_interaction_SPEC_VERSION = 2
HTC_VIVE_FOCUS3_CONTROLLER_INTERACTION_EXTENSION_NAME = "XR_HTC_vive_focus3_controller_interaction"
HTC_hand_interaction = 1
HTC_hand_interaction_SPEC_VERSION = 1
HTC_HAND_INTERACTION_EXTENSION_NAME = "XR_HTC_hand_interaction"
HTC_vive_wrist_tracker_interaction = 1
HTC_vive_wrist_tracker_interaction_SPEC_VERSION = 1
HTC_VIVE_WRIST_TRACKER_INTERACTION_EXTENSION_NAME = "XR_HTC_vive_wrist_tracker_interaction"
FB_color_space = 1
FB_color_space_SPEC_VERSION = 3
FB_COLOR_SPACE_EXTENSION_NAME = "XR_FB_color_space"
FB_hand_tracking_mesh = 1
FB_hand_tracking_mesh_SPEC_VERSION = 3
FB_HAND_TRACKING_MESH_EXTENSION_NAME = "XR_FB_hand_tracking_mesh"
FB_hand_tracking_aim = 1
FB_hand_tracking_aim_SPEC_VERSION = 2
FB_HAND_TRACKING_AIM_EXTENSION_NAME = "XR_FB_hand_tracking_aim"
FB_hand_tracking_capsules = 1
HAND_TRACKING_CAPSULE_POINT_COUNT_FB = 2
HAND_TRACKING_CAPSULE_COUNT_FB = 19
FB_hand_tracking_capsules_SPEC_VERSION = 3
FB_HAND_TRACKING_CAPSULES_EXTENSION_NAME = "XR_FB_hand_tracking_capsules"
FB_HAND_TRACKING_CAPSULE_POINT_COUNT = HAND_TRACKING_CAPSULE_POINT_COUNT_FB
FB_HAND_TRACKING_CAPSULE_COUNT = HAND_TRACKING_CAPSULE_COUNT_FB
FB_spatial_entity = 1
UUID_SIZE_EXT = 16
FB_spatial_entity_SPEC_VERSION = 1
FB_SPATIAL_ENTITY_EXTENSION_NAME = "XR_FB_spatial_entity"
FB_foveation = 1
FB_foveation_SPEC_VERSION = 1
FB_FOVEATION_EXTENSION_NAME = "XR_FB_foveation"
FB_foveation_configuration = 1
FB_foveation_configuration_SPEC_VERSION = 1
FB_FOVEATION_CONFIGURATION_EXTENSION_NAME = "XR_FB_foveation_configuration"
FB_keyboard_tracking = 1
FB_keyboard_tracking_SPEC_VERSION = 1
FB_KEYBOARD_TRACKING_EXTENSION_NAME = "XR_FB_keyboard_tracking"
MAX_KEYBOARD_TRACKING_NAME_SIZE_FB = 128
FB_triangle_mesh = 1
FB_triangle_mesh_SPEC_VERSION = 2
FB_TRIANGLE_MESH_EXTENSION_NAME = "XR_FB_triangle_mesh"
FB_passthrough = 1
FB_passthrough_SPEC_VERSION = 3
FB_PASSTHROUGH_EXTENSION_NAME = "XR_FB_passthrough"
PASSTHROUGH_COLOR_MAP_MONO_SIZE_FB = 256
FB_render_model = 1
NULL_RENDER_MODEL_KEY_FB = 0
FB_render_model_SPEC_VERSION = 3
FB_RENDER_MODEL_EXTENSION_NAME = "XR_FB_render_model"
MAX_RENDER_MODEL_NAME_SIZE_FB = 64
VARJO_foveated_rendering = 1
VARJO_foveated_rendering_SPEC_VERSION = 2
VARJO_FOVEATED_RENDERING_EXTENSION_NAME = "XR_VARJO_foveated_rendering"
VARJO_composition_layer_depth_test = 1
VARJO_composition_layer_depth_test_SPEC_VERSION = 2
VARJO_COMPOSITION_LAYER_DEPTH_TEST_EXTENSION_NAME = "XR_VARJO_composition_layer_depth_test"
VARJO_environment_depth_estimation = 1
VARJO_environment_depth_estimation_SPEC_VERSION = 1
VARJO_ENVIRONMENT_DEPTH_ESTIMATION_EXTENSION_NAME = "XR_VARJO_environment_depth_estimation"
VARJO_marker_tracking = 1
VARJO_marker_tracking_SPEC_VERSION = 1
VARJO_MARKER_TRACKING_EXTENSION_NAME = "XR_VARJO_marker_tracking"
VARJO_view_offset = 1
VARJO_view_offset_SPEC_VERSION = 1
VARJO_VIEW_OFFSET_EXTENSION_NAME = "XR_VARJO_view_offset"
ML_ml2_controller_interaction = 1
ML_ml2_controller_interaction_SPEC_VERSION = 1
ML_ML2_CONTROLLER_INTERACTION_EXTENSION_NAME = "XR_ML_ml2_controller_interaction"
MSFT_spatial_anchor_persistence = 1
MAX_SPATIAL_ANCHOR_NAME_SIZE_MSFT = 256
MSFT_spatial_anchor_persistence_SPEC_VERSION = 2
MSFT_SPATIAL_ANCHOR_PERSISTENCE_EXTENSION_NAME = "XR_MSFT_spatial_anchor_persistence"
ULTRALEAP_hand_tracking_forearm = 1
HAND_FOREARM_JOINT_COUNT_ULTRALEAP = 27
ULTRALEAP_hand_tracking_forearm_SPEC_VERSION = 1
ULTRALEAP_HAND_TRACKING_FOREARM_EXTENSION_NAME = "XR_ULTRALEAP_hand_tracking_forearm"
FB_spatial_entity_query = 1
FB_spatial_entity_query_SPEC_VERSION = 1
FB_SPATIAL_ENTITY_QUERY_EXTENSION_NAME = "XR_FB_spatial_entity_query"
FB_spatial_entity_storage = 1
FB_spatial_entity_storage_SPEC_VERSION = 1
FB_SPATIAL_ENTITY_STORAGE_EXTENSION_NAME = "XR_FB_spatial_entity_storage"
FB_space_warp = 1
FB_space_warp_SPEC_VERSION = 2
FB_SPACE_WARP_EXTENSION_NAME = "XR_FB_space_warp"
FB_scene = 1
FB_scene_SPEC_VERSION = 1
FB_SCENE_EXTENSION_NAME = "XR_FB_scene"
EXT_palm_pose = 1
EXT_palm_pose_SPEC_VERSION = 2
EXT_PALM_POSE_EXTENSION_NAME = "XR_EXT_palm_pose"
ALMALENCE_digital_lens_control = 1
ALMALENCE_digital_lens_control_SPEC_VERSION = 1
ALMALENCE_DIGITAL_LENS_CONTROL_EXTENSION_NAME = "XR_ALMALENCE_digital_lens_control"
FB_spatial_entity_container = 1
FB_spatial_entity_container_SPEC_VERSION = 2
FB_SPATIAL_ENTITY_CONTAINER_EXTENSION_NAME = "XR_FB_spatial_entity_container"
FB_passthrough_keyboard_hands = 1
FB_passthrough_keyboard_hands_SPEC_VERSION = 2
FB_PASSTHROUGH_KEYBOARD_HANDS_EXTENSION_NAME = "XR_FB_passthrough_keyboard_hands"
FB_composition_layer_settings = 1
FB_composition_layer_settings_SPEC_VERSION = 1
FB_COMPOSITION_LAYER_SETTINGS_EXTENSION_NAME = "XR_FB_composition_layer_settings"
META_performance_metrics = 1
META_performance_metrics_SPEC_VERSION = 1
META_PERFORMANCE_METRICS_EXTENSION_NAME = "XR_META_performance_metrics"
EXT_uuid = 1
EXT_uuid_SPEC_VERSION = 1
EXT_UUID_EXTENSION_NAME = "XR_EXT_uuid"
SPACE_VELOCITY_LINEAR_VALID_BIT = 0x00000001
SPACE_VELOCITY_ANGULAR_VALID_BIT = 0x00000002
SPACE_LOCATION_ORIENTATION_VALID_BIT = 0x00000001
SPACE_LOCATION_POSITION_VALID_BIT = 0x00000002
SPACE_LOCATION_ORIENTATION_TRACKED_BIT = 0x00000004
SPACE_LOCATION_POSITION_TRACKED_BIT = 0x00000008
SWAPCHAIN_CREATE_PROTECTED_CONTENT_BIT = 0x00000001
SWAPCHAIN_CREATE_STATIC_IMAGE_BIT = 0x00000002
SWAPCHAIN_USAGE_COLOR_ATTACHMENT_BIT = 0x00000001
SWAPCHAIN_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT = 0x00000002
SWAPCHAIN_USAGE_UNORDERED_ACCESS_BIT = 0x00000004
SWAPCHAIN_USAGE_TRANSFER_SRC_BIT = 0x00000008
SWAPCHAIN_USAGE_TRANSFER_DST_BIT = 0x00000010
SWAPCHAIN_USAGE_SAMPLED_BIT = 0x00000020
SWAPCHAIN_USAGE_MUTABLE_FORMAT_BIT = 0x00000040
SWAPCHAIN_USAGE_INPUT_ATTACHMENT_BIT_MND = 0x00000080
SWAPCHAIN_USAGE_INPUT_ATTACHMENT_BIT_KHR = 0x00000080
COMPOSITION_LAYER_CORRECT_CHROMATIC_ABERRATION_BIT = 0x00000001
COMPOSITION_LAYER_BLEND_TEXTURE_SOURCE_ALPHA_BIT = 0x00000002
COMPOSITION_LAYER_UNPREMULTIPLIED_ALPHA_BIT = 0x00000004
VIEW_STATE_ORIENTATION_VALID_BIT = 0x00000001
VIEW_STATE_POSITION_VALID_BIT = 0x00000002
VIEW_STATE_ORIENTATION_TRACKED_BIT = 0x00000004
VIEW_STATE_POSITION_TRACKED_BIT = 0x00000008
INPUT_SOURCE_LOCALIZED_NAME_USER_PATH_BIT = 0x00000001
INPUT_SOURCE_LOCALIZED_NAME_INTERACTION_PROFILE_BIT = 0x00000002
INPUT_SOURCE_LOCALIZED_NAME_COMPONENT_BIT = 0x00000004
DEBUG_UTILS_MESSAGE_SEVERITY_VERBOSE_BIT_EXT = 0x00000001
DEBUG_UTILS_MESSAGE_SEVERITY_INFO_BIT_EXT = 0x00000010
DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT = 0x00000100
DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT = 0x00001000
DEBUG_UTILS_MESSAGE_TYPE_GENERAL_BIT_EXT = 0x00000001
DEBUG_UTILS_MESSAGE_TYPE_VALIDATION_BIT_EXT = 0x00000002
DEBUG_UTILS_MESSAGE_TYPE_PERFORMANCE_BIT_EXT = 0x00000004
DEBUG_UTILS_MESSAGE_TYPE_CONFORMANCE_BIT_EXT = 0x00000008
OVERLAY_MAIN_SESSION_ENABLED_COMPOSITION_LAYER_INFO_DEPTH_BIT_EXTX = 0x00000001
COMPOSITION_LAYER_IMAGE_LAYOUT_VERTICAL_FLIP_BIT_FB = 0x00000001
COMPOSITION_LAYER_SECURE_CONTENT_EXCLUDE_LAYER_BIT_FB = 0x00000001
COMPOSITION_LAYER_SECURE_CONTENT_REPLACE_LAYER_BIT_FB = 0x00000002
HAND_TRACKING_AIM_COMPUTED_BIT_FB = 0x00000001
HAND_TRACKING_AIM_VALID_BIT_FB = 0x00000002
HAND_TRACKING_AIM_INDEX_PINCHING_BIT_FB = 0x00000004
HAND_TRACKING_AIM_MIDDLE_PINCHING_BIT_FB = 0x00000008
HAND_TRACKING_AIM_RING_PINCHING_BIT_FB = 0x00000010
HAND_TRACKING_AIM_LITTLE_PINCHING_BIT_FB = 0x00000020
HAND_TRACKING_AIM_SYSTEM_GESTURE_BIT_FB = 0x00000040
HAND_TRACKING_AIM_DOMINANT_HAND_BIT_FB = 0x00000080
HAND_TRACKING_AIM_MENU_PRESSED_BIT_FB = 0x00000100
SWAPCHAIN_CREATE_FOVEATION_SCALED_BIN_BIT_FB = 0x00000001
SWAPCHAIN_CREATE_FOVEATION_FRAGMENT_DENSITY_MAP_BIT_FB = 0x00000002
KEYBOARD_TRACKING_EXISTS_BIT_FB = 0x00000001
KEYBOARD_TRACKING_LOCAL_BIT_FB = 0x00000002
KEYBOARD_TRACKING_REMOTE_BIT_FB = 0x00000004
KEYBOARD_TRACKING_CONNECTED_BIT_FB = 0x00000008
KEYBOARD_TRACKING_QUERY_LOCAL_BIT_FB = 0x00000002
KEYBOARD_TRACKING_QUERY_REMOTE_BIT_FB = 0x00000004
TRIANGLE_MESH_MUTABLE_BIT_FB = 0x00000001
PASSTHROUGH_CAPABILITY_BIT_FB = 0x00000001
PASSTHROUGH_CAPABILITY_COLOR_BIT_FB = 0x00000002
PASSTHROUGH_CAPABILITY_LAYER_DEPTH_BIT_FB = 0x00000004
PASSTHROUGH_IS_RUNNING_AT_CREATION_BIT_FB = 0x00000001
PASSTHROUGH_LAYER_DEPTH_BIT_FB = 0x00000002
PASSTHROUGH_STATE_CHANGED_REINIT_REQUIRED_BIT_FB = 0x00000001
PASSTHROUGH_STATE_CHANGED_NON_RECOVERABLE_ERROR_BIT_FB = 0x00000002
PASSTHROUGH_STATE_CHANGED_RECOVERABLE_ERROR_BIT_FB = 0x00000004
PASSTHROUGH_STATE_CHANGED_RESTORED_ERROR_BIT_FB = 0x00000008
RENDER_MODEL_SUPPORTS_GLTF_2_0_SUBSET_1_BIT_FB = 0x00000001
RENDER_MODEL_SUPPORTS_GLTF_2_0_SUBSET_2_BIT_FB = 0x00000002
COMPOSITION_LAYER_SPACE_WARP_INFO_FRAME_SKIP_BIT_FB = 0x00000001
DIGITAL_LENS_CONTROL_PROCESSING_DISABLE_BIT_ALMALENCE = 0x00000001
COMPOSITION_LAYER_SETTINGS_NORMAL_SUPER_SAMPLING_BIT_FB = 0x00000001
COMPOSITION_LAYER_SETTINGS_QUALITY_SUPER_SAMPLING_BIT_FB = 0x00000002
COMPOSITION_LAYER_SETTINGS_NORMAL_SHARPENING_BIT_FB = 0x00000004
COMPOSITION_LAYER_SETTINGS_QUALITY_SHARPENING_BIT_FB = 0x00000008
PERFORMANCE_METRICS_COUNTER_ANY_VALUE_VALID_BIT_META = 0x00000001
PERFORMANCE_METRICS_COUNTER_UINT_VALUE_VALID_BIT_META = 0x00000002
PERFORMANCE_METRICS_COUNTER_FLOAT_VALUE_VALID_BIT_META = 0x00000004


__all__ = [
    "ALMALENCE_DIGITAL_LENS_CONTROL_EXTENSION_NAME",
    "ALMALENCE_digital_lens_control",
    "ALMALENCE_digital_lens_control_SPEC_VERSION",
    "COMPOSITION_LAYER_BLEND_TEXTURE_SOURCE_ALPHA_BIT",
    "COMPOSITION_LAYER_CORRECT_CHROMATIC_ABERRATION_BIT",
    "COMPOSITION_LAYER_IMAGE_LAYOUT_VERTICAL_FLIP_BIT_FB",
    "COMPOSITION_LAYER_SECURE_CONTENT_EXCLUDE_LAYER_BIT_FB",
    "COMPOSITION_LAYER_SECURE_CONTENT_REPLACE_LAYER_BIT_FB",
    "COMPOSITION_LAYER_SETTINGS_NORMAL_SHARPENING_BIT_FB",
    "COMPOSITION_LAYER_SETTINGS_NORMAL_SUPER_SAMPLING_BIT_FB",
    "COMPOSITION_LAYER_SETTINGS_QUALITY_SHARPENING_BIT_FB",
    "COMPOSITION_LAYER_SETTINGS_QUALITY_SUPER_SAMPLING_BIT_FB",
    "COMPOSITION_LAYER_SPACE_WARP_INFO_FRAME_SKIP_BIT_FB",
    "COMPOSITION_LAYER_UNPREMULTIPLIED_ALPHA_BIT",
    "DEBUG_UTILS_MESSAGE_SEVERITY_ERROR_BIT_EXT",
    "DEBUG_UTILS_MESSAGE_SEVERITY_INFO_BIT_EXT",
    "DEBUG_UTILS_MESSAGE_SEVERITY_VERBOSE_BIT_EXT",
    "DEBUG_UTILS_MESSAGE_SEVERITY_WARNING_BIT_EXT",
    "DEBUG_UTILS_MESSAGE_TYPE_CONFORMANCE_BIT_EXT",
    "DEBUG_UTILS_MESSAGE_TYPE_GENERAL_BIT_EXT",
    "DEBUG_UTILS_MESSAGE_TYPE_PERFORMANCE_BIT_EXT",
    "DEBUG_UTILS_MESSAGE_TYPE_VALIDATION_BIT_EXT",
    "DIGITAL_LENS_CONTROL_PROCESSING_DISABLE_BIT_ALMALENCE",
    "EPIC_VIEW_CONFIGURATION_FOV_EXTENSION_NAME",
    "EPIC_view_configuration_fov",
    "EPIC_view_configuration_fov_SPEC_VERSION",
    "EXTX_OVERLAY_EXTENSION_NAME",
    "EXTX_overlay",
    "EXTX_overlay_SPEC_VERSION",
    "EXT_CONFORMANCE_AUTOMATION_EXTENSION_NAME",
    "EXT_DEBUG_UTILS_EXTENSION_NAME",
    "EXT_DPAD_BINDING_EXTENSION_NAME",
    "EXT_EYE_GAZE_INTERACTION_EXTENSION_NAME",
    "EXT_HAND_JOINTS_MOTION_RANGE_EXTENSION_NAME",
    "EXT_HAND_TRACKING_EXTENSION_NAME",
    "EXT_HP_MIXED_REALITY_CONTROLLER_EXTENSION_NAME",
    "EXT_PALM_POSE_EXTENSION_NAME",
    "EXT_PERFORMANCE_SETTINGS_EXTENSION_NAME",
    "EXT_SAMSUNG_ODYSSEY_CONTROLLER_EXTENSION_NAME",
    "EXT_THERMAL_QUERY_EXTENSION_NAME",
    "EXT_UUID_EXTENSION_NAME",
    "EXT_VIEW_CONFIGURATION_DEPTH_RANGE_EXTENSION_NAME",
    "EXT_WIN32_APPCONTAINER_COMPATIBLE_EXTENSION_NAME",
    "EXT_conformance_automation",
    "EXT_conformance_automation_SPEC_VERSION",
    "EXT_debug_utils",
    "EXT_debug_utils_SPEC_VERSION",
    "EXT_dpad_binding",
    "EXT_dpad_binding_SPEC_VERSION",
    "EXT_eye_gaze_interaction",
    "EXT_eye_gaze_interaction_SPEC_VERSION",
    "EXT_hand_joints_motion_range",
    "EXT_hand_joints_motion_range_SPEC_VERSION",
    "EXT_hand_tracking",
    "EXT_hand_tracking_SPEC_VERSION",
    "EXT_hp_mixed_reality_controller",
    "EXT_hp_mixed_reality_controller_SPEC_VERSION",
    "EXT_palm_pose",
    "EXT_palm_pose_SPEC_VERSION",
    "EXT_performance_settings",
    "EXT_performance_settings_SPEC_VERSION",
    "EXT_samsung_odyssey_controller",
    "EXT_samsung_odyssey_controller_SPEC_VERSION",
    "EXT_thermal_query",
    "EXT_thermal_query_SPEC_VERSION",
    "EXT_uuid",
    "EXT_uuid_SPEC_VERSION",
    "EXT_view_configuration_depth_range",
    "EXT_view_configuration_depth_range_SPEC_VERSION",
    "EXT_win32_appcontainer_compatible",
    "EXT_win32_appcontainer_compatible_SPEC_VERSION",
    "FACIAL_EXPRESSION_EYE_COUNT_HTC",
    "FACIAL_EXPRESSION_LIP_COUNT_HTC",
    "FALSE",
    "FB_COLOR_SPACE_EXTENSION_NAME",
    "FB_COMPOSITION_LAYER_ALPHA_BLEND_EXTENSION_NAME",
    "FB_COMPOSITION_LAYER_IMAGE_LAYOUT_EXTENSION_NAME",
    "FB_COMPOSITION_LAYER_SECURE_CONTENT_EXTENSION_NAME",
    "FB_COMPOSITION_LAYER_SETTINGS_EXTENSION_NAME",
    "FB_DISPLAY_REFRESH_RATE_EXTENSION_NAME",
    "FB_FOVEATION_CONFIGURATION_EXTENSION_NAME",
    "FB_FOVEATION_EXTENSION_NAME",
    "FB_HAND_TRACKING_AIM_EXTENSION_NAME",
    "FB_HAND_TRACKING_CAPSULES_EXTENSION_NAME",
    "FB_HAND_TRACKING_CAPSULE_COUNT",
    "FB_HAND_TRACKING_CAPSULE_POINT_COUNT",
    "FB_HAND_TRACKING_MESH_EXTENSION_NAME",
    "FB_KEYBOARD_TRACKING_EXTENSION_NAME",
    "FB_PASSTHROUGH_EXTENSION_NAME",
    "FB_PASSTHROUGH_KEYBOARD_HANDS_EXTENSION_NAME",
    "FB_RENDER_MODEL_EXTENSION_NAME",
    "FB_SCENE_EXTENSION_NAME",
    "FB_SPACE_WARP_EXTENSION_NAME",
    "FB_SPATIAL_ENTITY_CONTAINER_EXTENSION_NAME",
    "FB_SPATIAL_ENTITY_EXTENSION_NAME",
    "FB_SPATIAL_ENTITY_QUERY_EXTENSION_NAME",
    "FB_SPATIAL_ENTITY_STORAGE_EXTENSION_NAME",
    "FB_SWAPCHAIN_UPDATE_STATE_EXTENSION_NAME",
    "FB_TRIANGLE_MESH_EXTENSION_NAME",
    "FB_color_space",
    "FB_color_space_SPEC_VERSION",
    "FB_composition_layer_alpha_blend",
    "FB_composition_layer_alpha_blend_SPEC_VERSION",
    "FB_composition_layer_image_layout",
    "FB_composition_layer_image_layout_SPEC_VERSION",
    "FB_composition_layer_secure_content",
    "FB_composition_layer_secure_content_SPEC_VERSION",
    "FB_composition_layer_settings",
    "FB_composition_layer_settings_SPEC_VERSION",
    "FB_display_refresh_rate",
    "FB_display_refresh_rate_SPEC_VERSION",
    "FB_foveation",
    "FB_foveation_SPEC_VERSION",
    "FB_foveation_configuration",
    "FB_foveation_configuration_SPEC_VERSION",
    "FB_hand_tracking_aim",
    "FB_hand_tracking_aim_SPEC_VERSION",
    "FB_hand_tracking_capsules",
    "FB_hand_tracking_capsules_SPEC_VERSION",
    "FB_hand_tracking_mesh",
    "FB_hand_tracking_mesh_SPEC_VERSION",
    "FB_keyboard_tracking",
    "FB_keyboard_tracking_SPEC_VERSION",
    "FB_passthrough",
    "FB_passthrough_SPEC_VERSION",
    "FB_passthrough_keyboard_hands",
    "FB_passthrough_keyboard_hands_SPEC_VERSION",
    "FB_render_model",
    "FB_render_model_SPEC_VERSION",
    "FB_scene",
    "FB_scene_SPEC_VERSION",
    "FB_space_warp",
    "FB_space_warp_SPEC_VERSION",
    "FB_spatial_entity",
    "FB_spatial_entity_SPEC_VERSION",
    "FB_spatial_entity_container",
    "FB_spatial_entity_container_SPEC_VERSION",
    "FB_spatial_entity_query",
    "FB_spatial_entity_query_SPEC_VERSION",
    "FB_spatial_entity_storage",
    "FB_spatial_entity_storage_SPEC_VERSION",
    "FB_swapchain_update_state",
    "FB_swapchain_update_state_SPEC_VERSION",
    "FB_triangle_mesh",
    "FB_triangle_mesh_SPEC_VERSION",
    "FREQUENCY_UNSPECIFIED",
    "GUID_SIZE_MSFT",
    "HAND_FOREARM_JOINT_COUNT_ULTRALEAP",
    "HAND_JOINT_COUNT_EXT",
    "HAND_TRACKING_AIM_COMPUTED_BIT_FB",
    "HAND_TRACKING_AIM_DOMINANT_HAND_BIT_FB",
    "HAND_TRACKING_AIM_INDEX_PINCHING_BIT_FB",
    "HAND_TRACKING_AIM_LITTLE_PINCHING_BIT_FB",
    "HAND_TRACKING_AIM_MENU_PRESSED_BIT_FB",
    "HAND_TRACKING_AIM_MIDDLE_PINCHING_BIT_FB",
    "HAND_TRACKING_AIM_RING_PINCHING_BIT_FB",
    "HAND_TRACKING_AIM_SYSTEM_GESTURE_BIT_FB",
    "HAND_TRACKING_AIM_VALID_BIT_FB",
    "HAND_TRACKING_CAPSULE_COUNT_FB",
    "HAND_TRACKING_CAPSULE_POINT_COUNT_FB",
    "HTCX_VIVE_TRACKER_INTERACTION_EXTENSION_NAME",
    "HTCX_vive_tracker_interaction",
    "HTCX_vive_tracker_interaction_SPEC_VERSION",
    "HTC_FACIAL_TRACKING_EXTENSION_NAME",
    "HTC_HAND_INTERACTION_EXTENSION_NAME",
    "HTC_VIVE_COSMOS_CONTROLLER_INTERACTION_EXTENSION_NAME",
    "HTC_VIVE_FOCUS3_CONTROLLER_INTERACTION_EXTENSION_NAME",
    "HTC_VIVE_WRIST_TRACKER_INTERACTION_EXTENSION_NAME",
    "HTC_facial_tracking",
    "HTC_facial_tracking_SPEC_VERSION",
    "HTC_hand_interaction",
    "HTC_hand_interaction_SPEC_VERSION",
    "HTC_vive_cosmos_controller_interaction",
    "HTC_vive_cosmos_controller_interaction_SPEC_VERSION",
    "HTC_vive_focus3_controller_interaction",
    "HTC_vive_focus3_controller_interaction_SPEC_VERSION",
    "HTC_vive_wrist_tracker_interaction",
    "HTC_vive_wrist_tracker_interaction_SPEC_VERSION",
    "HUAWEI_CONTROLLER_INTERACTION_EXTENSION_NAME",
    "HUAWEI_controller_interaction",
    "HUAWEI_controller_interaction_SPEC_VERSION",
    "INFINITE_DURATION",
    "INPUT_SOURCE_LOCALIZED_NAME_COMPONENT_BIT",
    "INPUT_SOURCE_LOCALIZED_NAME_INTERACTION_PROFILE_BIT",
    "INPUT_SOURCE_LOCALIZED_NAME_USER_PATH_BIT",
    "KEYBOARD_TRACKING_CONNECTED_BIT_FB",
    "KEYBOARD_TRACKING_EXISTS_BIT_FB",
    "KEYBOARD_TRACKING_LOCAL_BIT_FB",
    "KEYBOARD_TRACKING_QUERY_LOCAL_BIT_FB",
    "KEYBOARD_TRACKING_QUERY_REMOTE_BIT_FB",
    "KEYBOARD_TRACKING_REMOTE_BIT_FB",
    "KHR_BINDING_MODIFICATION_EXTENSION_NAME",
    "KHR_COMPOSITION_LAYER_COLOR_SCALE_BIAS_EXTENSION_NAME",
    "KHR_COMPOSITION_LAYER_CUBE_EXTENSION_NAME",
    "KHR_COMPOSITION_LAYER_CYLINDER_EXTENSION_NAME",
    "KHR_COMPOSITION_LAYER_DEPTH_EXTENSION_NAME",
    "KHR_COMPOSITION_LAYER_EQUIRECT2_EXTENSION_NAME",
    "KHR_COMPOSITION_LAYER_EQUIRECT_EXTENSION_NAME",
    "KHR_LOADER_INIT_EXTENSION_NAME",
    "KHR_SWAPCHAIN_USAGE_INPUT_ATTACHMENT_BIT_EXTENSION_NAME",
    "KHR_VISIBILITY_MASK_EXTENSION_NAME",
    "KHR_binding_modification",
    "KHR_binding_modification_SPEC_VERSION",
    "KHR_composition_layer_color_scale_bias",
    "KHR_composition_layer_color_scale_bias_SPEC_VERSION",
    "KHR_composition_layer_cube",
    "KHR_composition_layer_cube_SPEC_VERSION",
    "KHR_composition_layer_cylinder",
    "KHR_composition_layer_cylinder_SPEC_VERSION",
    "KHR_composition_layer_depth",
    "KHR_composition_layer_depth_SPEC_VERSION",
    "KHR_composition_layer_equirect",
    "KHR_composition_layer_equirect2",
    "KHR_composition_layer_equirect2_SPEC_VERSION",
    "KHR_composition_layer_equirect_SPEC_VERSION",
    "KHR_loader_init",
    "KHR_loader_init_SPEC_VERSION",
    "KHR_swapchain_usage_input_attachment_bit",
    "KHR_swapchain_usage_input_attachment_bit_SPEC_VERSION",
    "KHR_visibility_mask",
    "KHR_visibility_mask_SPEC_VERSION",
    "MAX_ACTION_NAME_SIZE",
    "MAX_ACTION_SET_NAME_SIZE",
    "MAX_API_LAYER_DESCRIPTION_SIZE",
    "MAX_API_LAYER_NAME_SIZE",
    "MAX_APPLICATION_NAME_SIZE",
    "MAX_CONTROLLER_MODEL_NODE_NAME_SIZE_MSFT",
    "MAX_ENGINE_NAME_SIZE",
    "MAX_EXTENSION_NAME_SIZE",
    "MAX_KEYBOARD_TRACKING_NAME_SIZE_FB",
    "MAX_LOCALIZED_ACTION_NAME_SIZE",
    "MAX_LOCALIZED_ACTION_SET_NAME_SIZE",
    "MAX_PATH_LENGTH",
    "MAX_RENDER_MODEL_NAME_SIZE_FB",
    "MAX_RESULT_STRING_SIZE",
    "MAX_RUNTIME_NAME_SIZE",
    "MAX_SPATIAL_ANCHOR_NAME_SIZE_MSFT",
    "MAX_STRUCTURE_NAME_SIZE",
    "MAX_SYSTEM_NAME_SIZE",
    "META_PERFORMANCE_METRICS_EXTENSION_NAME",
    "META_performance_metrics",
    "META_performance_metrics_SPEC_VERSION",
    "MIN_COMPOSITION_LAYERS_SUPPORTED",
    "MIN_HAPTIC_DURATION",
    "ML_ML2_CONTROLLER_INTERACTION_EXTENSION_NAME",
    "ML_ml2_controller_interaction",
    "ML_ml2_controller_interaction_SPEC_VERSION",
    "MND_HEADLESS_EXTENSION_NAME",
    "MND_SWAPCHAIN_USAGE_INPUT_ATTACHMENT_BIT_EXTENSION_NAME",
    "MND_headless",
    "MND_headless_SPEC_VERSION",
    "MND_swapchain_usage_input_attachment_bit",
    "MND_swapchain_usage_input_attachment_bit_SPEC_VERSION",
    "MSFT_COMPOSITION_LAYER_REPROJECTION_EXTENSION_NAME",
    "MSFT_CONTROLLER_MODEL_EXTENSION_NAME",
    "MSFT_FIRST_PERSON_OBSERVER_EXTENSION_NAME",
    "MSFT_HAND_INTERACTION_EXTENSION_NAME",
    "MSFT_HAND_TRACKING_MESH_EXTENSION_NAME",
    "MSFT_SCENE_UNDERSTANDING_EXTENSION_NAME",
    "MSFT_SCENE_UNDERSTANDING_SERIALIZATION_EXTENSION_NAME",
    "MSFT_SECONDARY_VIEW_CONFIGURATION_EXTENSION_NAME",
    "MSFT_SPATIAL_ANCHOR_EXTENSION_NAME",
    "MSFT_SPATIAL_ANCHOR_PERSISTENCE_EXTENSION_NAME",
    "MSFT_SPATIAL_GRAPH_BRIDGE_EXTENSION_NAME",
    "MSFT_UNBOUNDED_REFERENCE_SPACE_EXTENSION_NAME",
    "MSFT_composition_layer_reprojection",
    "MSFT_composition_layer_reprojection_SPEC_VERSION",
    "MSFT_controller_model",
    "MSFT_controller_model_SPEC_VERSION",
    "MSFT_first_person_observer",
    "MSFT_first_person_observer_SPEC_VERSION",
    "MSFT_hand_interaction",
    "MSFT_hand_interaction_SPEC_VERSION",
    "MSFT_hand_tracking_mesh",
    "MSFT_hand_tracking_mesh_SPEC_VERSION",
    "MSFT_scene_understanding",
    "MSFT_scene_understanding_SPEC_VERSION",
    "MSFT_scene_understanding_serialization",
    "MSFT_scene_understanding_serialization_SPEC_VERSION",
    "MSFT_secondary_view_configuration",
    "MSFT_secondary_view_configuration_SPEC_VERSION",
    "MSFT_spatial_anchor",
    "MSFT_spatial_anchor_SPEC_VERSION",
    "MSFT_spatial_anchor_persistence",
    "MSFT_spatial_anchor_persistence_SPEC_VERSION",
    "MSFT_spatial_graph_bridge",
    "MSFT_spatial_graph_bridge_SPEC_VERSION",
    "MSFT_unbounded_reference_space",
    "MSFT_unbounded_reference_space_SPEC_VERSION",
    "NO_DURATION",
    "NULL_CONTROLLER_MODEL_KEY_MSFT",
    "NULL_HANDLE",
    "NULL_PATH",
    "NULL_RENDER_MODEL_KEY_FB",
    "NULL_SYSTEM_ID",
    "OCULUS_ANDROID_SESSION_STATE_ENABLE_EXTENSION_NAME",
    "OCULUS_android_session_state_enable",
    "OCULUS_android_session_state_enable_SPEC_VERSION",
    "OVERLAY_MAIN_SESSION_ENABLED_COMPOSITION_LAYER_INFO_DEPTH_BIT_EXTX",
    "PASSTHROUGH_CAPABILITY_BIT_FB",
    "PASSTHROUGH_CAPABILITY_COLOR_BIT_FB",
    "PASSTHROUGH_CAPABILITY_LAYER_DEPTH_BIT_FB",
    "PASSTHROUGH_COLOR_MAP_MONO_SIZE_FB",
    "PASSTHROUGH_IS_RUNNING_AT_CREATION_BIT_FB",
    "PASSTHROUGH_LAYER_DEPTH_BIT_FB",
    "PASSTHROUGH_STATE_CHANGED_NON_RECOVERABLE_ERROR_BIT_FB",
    "PASSTHROUGH_STATE_CHANGED_RECOVERABLE_ERROR_BIT_FB",
    "PASSTHROUGH_STATE_CHANGED_REINIT_REQUIRED_BIT_FB",
    "PASSTHROUGH_STATE_CHANGED_RESTORED_ERROR_BIT_FB",
    "PERFORMANCE_METRICS_COUNTER_ANY_VALUE_VALID_BIT_META",
    "PERFORMANCE_METRICS_COUNTER_FLOAT_VALUE_VALID_BIT_META",
    "PERFORMANCE_METRICS_COUNTER_UINT_VALUE_VALID_BIT_META",
    "RENDER_MODEL_SUPPORTS_GLTF_2_0_SUBSET_1_BIT_FB",
    "RENDER_MODEL_SUPPORTS_GLTF_2_0_SUBSET_2_BIT_FB",
    "SPACE_LOCATION_ORIENTATION_TRACKED_BIT",
    "SPACE_LOCATION_ORIENTATION_VALID_BIT",
    "SPACE_LOCATION_POSITION_TRACKED_BIT",
    "SPACE_LOCATION_POSITION_VALID_BIT",
    "SPACE_VELOCITY_ANGULAR_VALID_BIT",
    "SPACE_VELOCITY_LINEAR_VALID_BIT",
    "SWAPCHAIN_CREATE_FOVEATION_FRAGMENT_DENSITY_MAP_BIT_FB",
    "SWAPCHAIN_CREATE_FOVEATION_SCALED_BIN_BIT_FB",
    "SWAPCHAIN_CREATE_PROTECTED_CONTENT_BIT",
    "SWAPCHAIN_CREATE_STATIC_IMAGE_BIT",
    "SWAPCHAIN_USAGE_COLOR_ATTACHMENT_BIT",
    "SWAPCHAIN_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT",
    "SWAPCHAIN_USAGE_INPUT_ATTACHMENT_BIT_KHR",
    "SWAPCHAIN_USAGE_INPUT_ATTACHMENT_BIT_MND",
    "SWAPCHAIN_USAGE_MUTABLE_FORMAT_BIT",
    "SWAPCHAIN_USAGE_SAMPLED_BIT",
    "SWAPCHAIN_USAGE_TRANSFER_DST_BIT",
    "SWAPCHAIN_USAGE_TRANSFER_SRC_BIT",
    "SWAPCHAIN_USAGE_UNORDERED_ACCESS_BIT",
    "TRIANGLE_MESH_MUTABLE_BIT_FB",
    "TRUE",
    "ULTRALEAP_HAND_TRACKING_FOREARM_EXTENSION_NAME",
    "ULTRALEAP_hand_tracking_forearm",
    "ULTRALEAP_hand_tracking_forearm_SPEC_VERSION",
    "UUID_SIZE_EXT",
    "VALVE_ANALOG_THRESHOLD_EXTENSION_NAME",
    "VALVE_analog_threshold",
    "VALVE_analog_threshold_SPEC_VERSION",
    "VARJO_COMPOSITION_LAYER_DEPTH_TEST_EXTENSION_NAME",
    "VARJO_ENVIRONMENT_DEPTH_ESTIMATION_EXTENSION_NAME",
    "VARJO_FOVEATED_RENDERING_EXTENSION_NAME",
    "VARJO_MARKER_TRACKING_EXTENSION_NAME",
    "VARJO_QUAD_VIEWS_EXTENSION_NAME",
    "VARJO_VIEW_OFFSET_EXTENSION_NAME",
    "VARJO_composition_layer_depth_test",
    "VARJO_composition_layer_depth_test_SPEC_VERSION",
    "VARJO_environment_depth_estimation",
    "VARJO_environment_depth_estimation_SPEC_VERSION",
    "VARJO_foveated_rendering",
    "VARJO_foveated_rendering_SPEC_VERSION",
    "VARJO_marker_tracking",
    "VARJO_marker_tracking_SPEC_VERSION",
    "VARJO_quad_views",
    "VARJO_quad_views_SPEC_VERSION",
    "VARJO_view_offset",
    "VARJO_view_offset_SPEC_VERSION",
    "VERSION_1_0",
    "VIEW_STATE_ORIENTATION_TRACKED_BIT",
    "VIEW_STATE_ORIENTATION_VALID_BIT",
    "VIEW_STATE_POSITION_TRACKED_BIT",
    "VIEW_STATE_POSITION_VALID_BIT",
]



import ctypes
import pkg_resources
import platform

if platform.system() == "Windows":
    library_name = "openxr_loader.dll"
elif platform.system() == "Linux":
    library_name = "libopenxr_loader.so"
else:
    raise NotImplementedError

library_path = pkg_resources.resource_filename("xr.library", library_name)
openxr_loader_library = ctypes.cdll.LoadLibrary(library_path)

__all__ = [
    "openxr_loader_library",
]



import platform

__all__ = []

if platform.system() == "Windows":
    from .windows import *
    from . import windows
    __all__ += windows.__all__
elif platform.system() == "Linux":
    from .linux import *
    from . import linux
    __all__ += linux.__all__
else:
    raise NotImplementedError



import ctypes
from ctypes import Array
import enum
import platform
from typing import Sequence

# TODO: separate package for opengl stuff
import xr

if platform.system() == "Windows":
    from OpenGL import WGL
elif platform.system() == "Linux":
    from OpenGL import GLX
import glfw

from .enums import *
from .typedefs import *
from .functions import *
from .platform import *
from .version import *
from .exception import *


class Eye(enum.IntEnum):
    LEFT = 0
    RIGHT = 1


class InstanceObject(object):
    def __init__(
            self,
            enabled_extensions: Sequence[str] = None,
            application_name: str = None,
            application_version: Version = None,
            engine_name: str = None,
            engine_version: Version = None,
            next=None,
    ) -> None:
        if enabled_extensions is None:
            discovered_extensions = enumerate_instance_extension_properties()
            # Use the most reasonable default
            if KHR_OPENGL_ENABLE_EXTENSION_NAME in discovered_extensions:
                enabled_extensions = [KHR_OPENGL_ENABLE_EXTENSION_NAME, ]
            else:
                enabled_extensions = []
        if application_name is None:
            application_name = "Unknown application"
        self.application_name = application_name
        if application_version is None:
            application_version = Version()
        if engine_name is None:
            engine_name = "pyopenxr"
            engine_version = PYOPENXR_CURRENT_API_VERSION
        if engine_version is None:
            engine_version = Version()
        if application_version is None:
            application_version = xr.Version(0, 0, 0)
        application_info = ApplicationInfo(
            application_name=application_name,
            application_version=application_version.number(),
            engine_name=engine_name,
            engine_version=engine_version.number(),
            api_version=XR_CURRENT_API_VERSION,
        )
        instance_create_info = InstanceCreateInfo(
            create_flags=InstanceCreateFlags(),
            application_info=application_info,
            enabled_api_layer_names=[],
            enabled_extension_names=enabled_extensions,
            next=next,
        )
        self.handle = create_instance(instance_create_info)

    def __enter__(self):
        return self

    def __exit__(self, exception_type, value, traceback):
        self.destroy()

    def destroy(self):
        if self.handle is not None:
            destroy_instance(self.handle)
            self.handle = None

    def get_properties(self) -> InstanceProperties:
        return xr.get_instance_properties(instance=self.handle)


class SystemObject(object):
    def __init__(
            self,
            instance: InstanceObject,
            form_factor: FormFactor = FormFactor.HEAD_MOUNTED_DISPLAY,
    ) -> None:
        # TODO: default managed value for instance
        system_get_info = SystemGetInfo(
            form_factor=form_factor,
        )
        self.id = get_system(instance.handle, system_get_info)
        self.instance = instance

    def __enter__(self):
        return self

    def __exit__(self, exception_type, value, traceback):
        self.id = None


class GlfwWindow(object):
    def __init__(
            self,
            system: SystemObject,
            title: str = None,
            mirror_window: bool = False
    ) -> None:
        if title is None:
            title = system.instance.application_name
        if not glfw.init():
            raise XrException("GLFW initialization failed")
        if mirror_window:
            self.window_size = [s // 4 for s in system.render_target_size]
        else:
            self.window_size = (64, 64)
            glfw.window_hint(glfw.VISIBLE, False)
        self.system = system
        self.pxrGetOpenGLGraphicsRequirementsKHR = ctypes.cast(
            get_instance_proc_addr(
                self.system.instance.handle,
                "xrGetOpenGLGraphicsRequirementsKHR",
            ),
            PFN_xrGetOpenGLGraphicsRequirementsKHR
        )
        self.graphics_requirements = GraphicsRequirementsOpenGLKHR()  # TODO: others
        result = self.pxrGetOpenGLGraphicsRequirementsKHR(
            self.system.instance.handle,
            self.system.id,
            ctypes.byref(self.graphics_requirements))  # TODO: pythonic wrapper
        result = check_result(Result(result))
        if result.is_exception():
            raise result
        glfw.window_hint(glfw.DOUBLEBUFFER, False)
        glfw.window_hint(glfw.CONTEXT_VERSION_MAJOR, 4)
        glfw.window_hint(glfw.CONTEXT_VERSION_MINOR, 5)
        glfw.window_hint(glfw.OPENGL_PROFILE, glfw.OPENGL_CORE_PROFILE)
        self.window = glfw.create_window(*self.window_size, title, None, None)
        if self.window is None:
            raise XrException("Failed to create GLFW window")
        glfw.make_context_current(self.window)
        # Attempt to disable vsync on the desktop window, or
        # it will interfere with the OpenXR frame loop timing
        glfw.swap_interval(0)
        self.graphics_binding = None
        if platform.system() == "Windows":
            self.graphics_binding = GraphicsBindingOpenGLWin32KHR()
            self.graphics_binding.h_dc = WGL.wglGetCurrentDC()
            self.graphics_binding.h_glrc = WGL.wglGetCurrentContext()
        elif platform.system() == "Linux":
            drawable = GLX.glXGetCurrentDrawable()
            context = GLX.glXGetCurrentContext()
            display = GLX.glXGetCurrentDisplay()
            self.graphics_binding = GraphicsBindingOpenGLXlibKHR(
                x_display=display,
                glx_drawable=drawable,
                glx_context=context,
            )
        else:
            raise NotImplementedError

    def __enter__(self):
        return self

    def __exit__(self, exception_type, value, traceback):
        glfw.terminate()


class SessionObject(object):
    def __init__(self, system: SystemObject, graphics_binding):
        graphics_binding_pointer = None
        if graphics_binding is not None:
            graphics_binding_pointer = ctypes.cast(
                ctypes.pointer(graphics_binding),
                ctypes.c_void_p)
        session_create_info = SessionCreateInfo(
            next=graphics_binding_pointer,
            create_flags=SessionCreateFlags(),
            system_id=system.id,
        )
        self.handle = create_session(
            system.instance.handle,
            session_create_info
        )
        self.state = SessionState.IDLE
        self.frame_state = FrameState()
        self.system = system
        self.space = SpaceObject(self)
        self.view_configuration_type = ViewConfigurationType.PRIMARY_STEREO

    def __enter__(self):
        return self

    def __exit__(self, type_, value, traceback):
        self.destroy()

    def begin_frame(self):
        begin_frame(self.handle)

    def destroy(self):
        if self.handle is None:
            return
        try:
            destroy_session(self.handle)
        finally:
            self.handle = None

    def end_frame(self, layers=None):
        frame_end_info = FrameEndInfo(
            display_time=self.frame_state.predicted_display_time,
            environment_blend_mode=EnvironmentBlendMode.OPAQUE,
            layers=layers,
        )
        end_frame(self.handle, frame_end_info)

    def locate_views(self) -> (ViewState, Array):
        view_configuration_type = self.view_configuration_type
        # TODO: put this someplace else
        # TODO: if self.state....
        display_time = self.frame_state.predicted_display_time
        #
        view_locate_info = ViewLocateInfo(
            view_configuration_type,
            display_time,
            self.space.handle,
        )
        return locate_views(self.handle, view_locate_info)

    def on_state_changed(self, session_state_changed_event):
        if self.handle is None:
            return
        if not StructureType(session_state_changed_event.type) == StructureType.EVENT_DATA_SESSION_STATE_CHANGED:
            return
        event = ctypes.cast(
            ctypes.byref(session_state_changed_event),
            ctypes.POINTER(EventDataSessionStateChanged)).contents
        self.state = SessionState(event.state)
        if self.state == SessionState.READY:
            if self.handle is not None:
                sbi = SessionBeginInfo(self.view_configuration_type)
                begin_session(self.handle, sbi)
        elif self.state == SessionState.STOPPING:
            self.destroy()

    def poll_xr_events(self):
        while True:
            try:
                event_buffer = poll_event(self.system.instance.handle)
                event_type = StructureType(event_buffer.type)
                if event_type == StructureType.EVENT_DATA_SESSION_STATE_CHANGED:
                    self.on_state_changed(event_buffer)
            except EventUnavailable:
                break

    def wait_frame(self):
        self.frame_state = wait_frame(self.handle)


class SpaceObject(object):
    def __init__(
            self,
            session: SessionObject,
            reference_space_type: ReferenceSpaceType = ReferenceSpaceType.STAGE,
            pose_in_reference_space: Posef = None,
    ):
        if pose_in_reference_space is None:
            pose_in_reference_space = Posef()
        reference_space_create_info = ReferenceSpaceCreateInfo(
            reference_space_type=reference_space_type,
            pose_in_reference_space=pose_in_reference_space,
        )
        self.handle = create_reference_space(session.handle, reference_space_create_info)


__all__ = [
    "Eye",
    "GlfwWindow",
    "InstanceObject",
    "SessionObject",
    "SpaceObject",
    "SystemObject",
]



import platform

from .layer_path import add_folder_to_api_layer_path, expose_packaged_api_layers

from .loader_interfaces import *
from .dynamic_api_layer_base import *
from . import loader_interfaces, dynamic_api_layer_base  # For __all__ handling

# Expose symbolic names for packaged api layers
LUNARG_api_dump_APILAYER_NAME = "XR_APILAYER_LUNARG_api_dump"
LUNARG_core_validation_APILAYER_NAME = "XR_APILAYER_LUNARG_core_validation"

# Automatically expose packaged API layers for supported platforms
if platform.system() in ["Windows", "Linux"]:
    expose_packaged_api_layers()


__all__ = [
    "expose_packaged_api_layers",
    "LUNARG_api_dump_APILAYER_NAME",
    "LUNARG_core_validation_APILAYER_NAME",
]
__all__ += loader_interfaces.__all__
__all__ += dynamic_api_layer_base.__all__



import ctypes

from .layer_path import py_layer_library_path
from ..exception import Result
from .loader_interfaces import NegotiateLoaderInfo, NegotiateApiLayerRequest, PFN_xrNegotiateLoaderApiLayerInterface

_python_layer_library = ctypes.cdll.LoadLibrary(py_layer_library_path())

xrNegotiateLoaderApiLayerInterface = _python_layer_library.xrNegotiateLoaderApiLayerInterface
xrNegotiateLoaderApiLayerInterface.restype = Result
xrNegotiateLoaderApiLayerInterface.argtypes = [
    ctypes.POINTER(NegotiateLoaderInfo),  # loaderInfo
    ctypes.c_char_p,  # layerName
    ctypes.POINTER(NegotiateApiLayerRequest),  # apiLayerRequest
]

insertXrApiLayer = _python_layer_library.insertXrApiLayer
insertXrApiLayer.restype = None
insertXrApiLayer.argtypes = [
    ctypes.c_char_p,  # pName
    ctypes.c_int,  # nameLength
    PFN_xrNegotiateLoaderApiLayerInterface,  # negotiateFunction
]
